//*****************************************************************************/
// ADC Driver and data collection routines
//*****************************************************************************/

#include	<cfxbios.h>		// Persistor BIOS and I/O Definitions
#include	<cfxpico.h>		// Persistor PicoDOS Definitions
#include	<assert.h>
#include	<ctype.h>
#include	<errno.h>
#include	<float.h>
#include	<limits.h>
#include	<locale.h>
#include	<math.h> 
#include	<setjmp.h>
#include	<signal.h>
#include	<stdarg.h>
#include	<stddef.h>
#include	<stdio.h> 
#include	<stdlib.h>
#include	<time.h>
#include 	<string.h>


#include   "IODP_CalConstant.h"
#include   "CS5534Lib.h"
#include   "IODP_Paro.h"

#include   "IODP_ADC.h"




#define ADQMAX  20
typedef struct ADQ{
	unsigned long qsec;
	unsigned short qtick;
 	long cval1;
 	long cval2;
 	long cval3;
 	long cval4;
 	long cval5;
 	long cval6;
 	long cval7;
 	long cval8;
}  ADQElem;

long cval1,cval2,cval3, cval4, cval5, cval6, cval7, cval8;


static long adtotal;
static short ADQinidx;
static short ADQoutidx;
static short ADQlength;

bool UpdateFlag = false;
static FILE *adfile;

static ADQElem   ADQData[ADQMAX];

static ADQElem *UpdateQElem = NULL;
static unsigned char adslot = 4;

static bool ADActive = false;
static char *ppstr;


float BrdTemp(float res);
float TipTemp(float res);
float VBattery(long bcount);

//  accelerometers use pcs2
//  ADC 1 uses PCS 1
//  ADC 2 uses PCS 2
//*****************************************************************************/

void InitADSPI(short slot){
	adslot = slot;

	// call the CS5534 library to initialize the ADC
	CS5534Init(slot);
	SetAllSpeeds(3);  // 60 hz unless otherwise specified

 	SetSpeed(2,2);// set channels to 30Hz.	
 	SetSpeed(3,4);
 	SetSpeed(4,4);
 	SetSpeed(5,4);
 	SetSpeed(6,4);
 	SetSpeed(7,4);
 	SetSpeed(8,4);

	SetBipolar(1,false);  // set all channels to unipolar
	SetBipolar(2,false);
	SetBipolar(3,false);
	SetBipolar(4,false);
	SetBipolar(5,true);   //Pressure channels are bipolar - K. Darnell 04/10
	SetBipolar(6,true);
	SetBipolar(7,false);
	SetBipolar(8,false);

	SetGain(1,8);  // board temp has gain 8
	//SetGain(2,4);  // tip temp has gain 4	
	SetGain(4,2);  // vBat has gain 2
	
	
	// set up the ADC registers with calibration constants
	// read from file IODP_CalConstant.h.
	CS5534SetGainCal(1,BRDCALSCALE);
	CS5534SetOffCal(1,BRDCALOFFSET);
		
	//CS5534SetGainCal(2,TIPCALSCALE);
	//CS5534SetOffCal(2,TIPCALOFFSET);
		
	CS5534SetGainCal(4,BATCALSCALE);
	CS5534SetOffCal(4,BATCALOFFSET);
	

	// initialize the ADC Queue also
	ADQinidx = 0;
	ADQoutidx = 0;
	ADQlength = 0;
	
	InitPressure();
	ADActive = false;

}



void ShowADRegisters(void){

	ShowSetup();
		
}

void ShowADCValue(short chan){
long adval;
float res;
	if((chan < 1) || (chan > 8)) return;

	adval = CS5534GetValue(chan);
	switch(chan){
		case  1 : res = RBRDSCALE * adval/FS24 * 2 + RBRDOFFSET;; 
				  printf("Board R = %8.2f\n", res);
				break;
		case  2 : 
		          res = adval;
				//res = RTIPSCALE * adval/FS24 + RTIPOFFSET;; 
				  printf("Tip  Value = %8.2f\n", res);  
		
				break;
		case  3 :  res = adval;
                   printf("Press = %10.3f\n", res);	
				break;
															
		case  4 :  res = VBATSCALE * adval/FS24 + VBATOFFSET;
				   printf("Battery Volts:  %6.2f\n", res);
				break;
					
		case  5 :   res = adval;
		            printf("Value = %10.3f\n", res);
		
				break;
					
		case  6 :   res = adval;
		            printf("Value = %10.3f\n", res);
		
				break;
		case  7 :   res = adval;
		            printf("Value = %10.3f\n", res);
		
				break;
		case  8 :   res = adval;
		            printf("Value = %10.3f\n", res);
		
				break;			
							
	}	

}


//  Called from PIT Chore  reads the ADC results and
//  stores in the global variables. Also puts data in ADQueue
//  ADC collection can't use any of the  tick50 values 
//  evenly divisible by 5 as those are used for Accelerometer collection
void ReadADC(short tick50){

static ADQElem *aqp;

static unsigned long secs;
static unsigned short ticks;
	if(ADActive){  // don't collect unless flag is set
		switch(tick50){
			case   1: CS5534SetPower(POWERNORMAL,0);   //Turn on ADC-U5 - K. Darnell 04/10     
                      
					  RTCGetTime(&secs, &ticks);
					  if(ADQlength < ADQMAX){
							aqp = &ADQData[ADQinidx];							

							aqp->qsec = secs;	
							aqp->qtick = ticks;	
						}	else aqp = NULL;				  	

			          break;
			case   2:   //request channel 2 tip temp
						CS5534RequestData(2); 
						break;         	
			case   9:   // collect channel 2 tip temp
						cval2 = CS5534Result(2);
					 break;
		      	
			case   11:      //request channel 1 board temp
						CS5534RequestData(1);
			          break;				
			case  16:         //collect channel 1 board temp
					     cval1 = CS5534Result(1);			
			          break;				
			case   18:         //request channel 3 pressure reading
					  CS5534RequestData(3);			          
			 		 break;	
			case   21:         //collect channel 3 pressure reading
					 cval3 = CS5534Result(3);			
			         break;
		
			case   22:         //request channel 4 battery voltage
					  CS5534RequestData(4);			
			         break;
			case   24:   //  collect channel 4 battery voltage
					    cval4 = CS5534Result(4);	
					 break;  
			case   27:	CS5534SetPower(POWERSAVE,0); //Turn off ADC-U5 - K. Darnell 04/10
			         break;
			case   28:  CS5534SetPower(POWERNORMAL,1);  //Turn on ADC-U7 - K. Darnell 04/10
			         break;           	 
			             	
			case   29:   // request channel 5 reading
						CS5534RequestData(5);	
					 break; 	
			
			case   31:    // collect channel 5 reading
			            cval5 = CS5534Result(5);
			          break;
			case   33:   // request channel 5 reading
						CS5534RequestData(6);	
					 break; 	
			
			case   36:    // collect channel 6 reading
			            cval6 = CS5534Result(6);
			          break;
			case   38:   // request channel 7 reading
						CS5534RequestData(7);	
					 break; 	
			
			case   41:    // collect channel 7 reading
			            cval7 = CS5534Result(7);
			          break;
			case   43:   // request channel 8 reading
						CS5534RequestData(8);	
					 break; 	
			
			case   46:    // collect channel 8 reading
			            cval8 = CS5534Result(8);
			          break;
			                        
		   case   48: // now put the  ADC and pressure values into the queue
						if(aqp != NULL){
							aqp->cval1 = cval1;
							aqp->cval2 = cval2;
							aqp->cval3 = cval3;
							aqp->cval4 = cval4;
							aqp->cval5 = cval5;
							aqp->cval6 = cval6;	
							aqp->cval7 = cval7;	
							aqp->cval8 = cval8;		
							ADQinidx++;
							if(ADQinidx >= ADQMAX) ADQinidx = 0;
							ADQlength++;
			case   49: CS5534SetPower(POWERSAVE,1); //Turn off ADC-U7 - K. Darnell 04/10
			          break; 
							
			
																					
						}
		 } // end of switch(tick50).....
	} // end of if(ADActive)...
}


//  called by foreground loop to pull queue elements and write to file
void CheckADCQueue(void){
register ADQElem *aqp;
	if(ADQlength > 0){
		aqp = &ADQData[ADQoutidx];
		if(adfile != NULL){
			//fwrite(aqp, 1,sizeof(ADQElem), adfile);
		}
		ADQoutidx++;
		if(ADQoutidx >= ADQMAX) ADQoutidx = 0;
		ADQlength--;
		adtotal++;	
		UpdateQElem = aqp;
		UpdateFlag = true;	
	}
}

void ADCStart(void){
	ADQinidx = 0;
	ADQoutidx = 0;
	ADQlength = 0;
	adtotal = 0;
	ADActive = false;
	//adfile = fopen("adtemp.dat", "wb");
	//cprintf("Opened adtemp.dat\n");
	ADActive = true;

}

void ADCStop(void){
	ADActive = false;
	if(adfile != NULL){
		fclose(adfile);
		cprintf("Closed adtemp.dat.   %ld values were written\n", adtotal);
		adfile = NULL;
	}

}

bool ADCOpenPlayback(void){


	ADCStop();
	adfile = fopen("adtemp.dat", "rb");
	if(adfile == NULL){
		cprintf("\nCan't open adtemp.dat\n");
		return false;		
	} else return true;

}

time_t ADFileStart(void){
time_t tf;
ADQElem aqe;
int numread;
FILE *f;
	f = fopen("adtemp.dat", "rb");
	tf = 0;
	if(f == NULL){
		cprintf("\nCan't open adtemp.dat\n");
		return false;		
	} else {
		numread = fread(&aqe, 1, sizeof(aqe), f);
		if(numread == sizeof(aqe)){
			tf =  aqe.qsec;
		} else tf = 0l;
		fclose(f);
	}
	return tf;
}

// return a string with the ADC portion of a playback
char  *ADCGetPlayback(void){
static char pbs[100];
char tms[30];
char nums[50];
short tsec, numread;
float rtip,rbrd,bvolts,ttip,tbrd,ch3,ch5,ch6,ch7,ch8,tip;
struct tm *tmp;
ADQElem aqe;
	PIOSet(25);

	if(adfile == NULL){ // error exit if file not open
		pbs[0] = 0;
		return &pbs[0];
	}
	

	if(!feof(adfile) && !kbhit()){
		numread = fread(&aqe, 1, sizeof(aqe), adfile);
		if(numread == sizeof(aqe)){
			tsec =  (aqe.qtick+200)/400;
			tmp =localtime(&aqe.qsec);
			tip = aqe.cval2;
			rtip = RTIPSCALE * aqe.cval2/FS24 + RTIPOFFSET;
			rbrd = RBRDSCALE * aqe.cval1/FS24 * 2 + RBRDOFFSET;
			bvolts = VBATSCALE * aqe.cval4/FS24 + VBATOFFSET;
		  	tbrd =  BrdTemp(rbrd);
			ttip =  TipTemp(rtip);
			ch3 = aqe.cval3;
			ch5 = aqe.cval5; 
			ch6 = aqe.cval6;
			ch7 = aqe.cval7;
			ch8 = aqe.cval8;
			strftime(tms, 31," %H:%M:%S",tmp);	
#ifdef USECOMMAS
			sprintf(pbs,"%s.%02d, ",tms,tsec);
			sprintf(nums,"%10.0f %6.1f %6.1f %10.0f %10.0f", tip, tbrd, bvolts, ch5, ch6);
#else 
			sprintf(pbs,"%s.%02d ",tms,tsec);
			sprintf(nums,"%10.0f %6.1f  %6.1f %10.0f %10.0f", tip, tbrd, bvolts, ch5, ch6);
#endif			


			strcat(pbs,nums);
        } else {
            pbs[0] = 0;
        }
	
	} else {
		pbs[0] = 0;
		fclose(adfile);
		adfile = NULL;
	}

	PIOClear(25);
	RTCDelayMicroSeconds(200);

	return &pbs[0];
}

void ADCPlayback(void){

char *pbs;
bool done = false;

	//ADCStop();
	
	if(ADCOpenPlayback()){
	
		cprintf("\n");  // put header here
		while(!done){
			pbs = ADCGetPlayback();
			if(pbs[0] == 0) done = true;
			printf("%s\n",pbs);
		}	
	} else {
		cprintf("\nCan't open adtemp.dat\n");	
	}
	cprintf("\n");

}
void TestPressure(void){

	RequestPressure();
	RTCDelayMicroSeconds(2000000l);
	ParsePressure();

}

float BrdTemp(float res){
float  lnr, tinv;
	lnr = log(res);	
	tinv = BTA + BTB * lnr + BTC *lnr*lnr*lnr;
	return 1.0/tinv - 273.15;

}

float TipTemp(float res){
float   lnr, tinv;
	lnr = log(res);	
	tinv = (TTA + TTB * lnr + TTC *lnr*lnr*lnr);
	
	return 1.0/tinv -273.15;
}


float VBattery(long bcount){

	return VBATSCALE * bcount/FS24 + VBATOFFSET;

}



float GetVolts(void){
long adval;
float vbat;
	adval = CS5534GetValue(4);
	vbat = VBattery(adval);
	//printf("vbat: %6.3f\n", vbat);
	return vbat;
}

//  *************************************************
//  Called by main loop when UpdateFlag is true
void ShowADC(char *b){
static char pbs[100];
char tms[30];

short tsec;
float rtip,rbrd,bvolts,ch3,ch5,ch6,ch7,ch8,tip;
float ttip,tbrd;
struct tm *tmp;
ADQElem *aqp;
FILE *f;

	//Make data available to Hyperterminal screen - K. Darnell 04/10
	f=fopen(b,"a");
	fflush(f);
	if(UpdateQElem == NULL) return;
	aqp = UpdateQElem;
	tsec =  (aqp->qtick+200)/400;
	tmp =localtime(&aqp->qsec);
	tip = aqp->cval2;
	rtip = RTIPSCALE * aqp->cval2/FS24 + RTIPOFFSET;
	rbrd = RBRDSCALE * aqp->cval1/FS24 + RBRDOFFSET;

	tbrd =  BrdTemp(rbrd);
	ttip =  TipTemp(rtip);
	bvolts = VBattery(aqp->cval4);
	ch3 = aqp->cval3;
	ch5 = aqp->cval5;
	ch6 = aqp->cval6;
	ch7 = aqp->cval7;
	ch8 = aqp->cval8;
	
	strftime(tms, 31,"%m%d%Y %H:%M:%S",tmp);	
	printf("#%s.%02d   ",tms,tsec);
	printf("%10.0f %6.1f %4.1f %10.0f %10.0f\n", tip, tbrd, bvolts, ch5, ch6);
	
	//Print to file - K. Darnell 04/10
	fprintf(f,"%s.%02d   ",tms,tsec);
	fprintf(f,"%10.0f %6.1f %4.1f %10.0f %10.0f\n", tip, tbrd, bvolts, ch5, ch6);
	fclose(f);
	UpdateFlag = false;

}
