//*****************************************************************************/
// Accelerometer Driver and data collection routines
//*****************************************************************************/

#include	<cfxbios.h>		// Persistor BIOS and I/O Definitions
#include	<cfxpico.h>		// Persistor PicoDOS Definitions
#include	<assert.h>
#include	<ctype.h>
#include	<errno.h>
#include	<float.h>
#include	<limits.h>
#include	<locale.h>
#include	<math.h> 
#include	<setjmp.h>
#include	<signal.h>
#include	<stdarg.h>
#include	<stddef.h>
#include	<stdio.h> 
#include	<stdlib.h>
#include	<time.h>

#include   "IODP_Accel.h"
#include   "IODP_CalConstant.h"

#define OUTX_L  0x28
#define OUTX_H  0x29
#define OUTY_L  0x2A
#define OUTY_H  0x2B
#define OUTZ_L  0x2C
#define OUTZ_H  0x2D

#define AXQMAX  20
typedef struct AXQ{
	unsigned long qsec;
	unsigned short qtick;
	short qax;
	short qay;
	short qaz;

}  AXQElem;

unsigned short ax,ay,az;

static long axtotal;
static short AXQinidx;
static short AXQoutidx;
static short AXQlength;

static FILE *axfile;

static AXQElem   AXQData[AXQMAX];

static unsigned char axslot = 4;
static uspv cmdptr;	// location to store command byte to transmit
static uspv xmtdatptr;		// address of short word to transmit A/D
static uspv rcvdatptr;		// address of short word returned from A/D
static ucpv  cmdRamPtr;    //  command RAM is array of bytes
static bool AXActive = false;


unsigned char ReadMoreReg(unsigned char reg);

#define CPOL 0x0000
#define CPHA 0x0000

//#define CPOL 0x0200
//#define CPHA 0x0100
//  accelerometers use pcs2
//*****************************************************************************/

void InitAXSPI(short slot){
	axslot = slot;
	*SPCR1 = 0x000;		// disable the SPI
	*QPDR = 0xFF;		// set , SCK high when not used 
	*QPAR = 0xFF;  		// set all as QSM pins 
	*QDDR = 0xFE; 		// set pin 0, as input 


	*SPCR0 = 0x8006 | CPOL | CPHA; 	// sets master, 16 bits  baud divisor 8, CPHA 1
						
	*SPCR2 = 0;			
	*SPCR3 = 0; 		// halt not enabled, no loop back
	SPICMDARRAY[slot] = 0x4B; // 16-bit transfer, PCS2  used
 	*SPCR1 = 0x8000;    // Enable now
 	Delay2us();
 	while(*SPSR < 128){ }; // wait until qsm done flag set

// we  set one QSPI register starting at slot	value
// which is normally 4 for demo programs
	cmdptr  =	((ushort*) (IMBASE + 0xD20 + (slot *2)));
	xmtdatptr 	= 	((ushort *) (IMBASE + 0xD20 + (slot *2)));
	rcvdatptr 	=	((ushort*)  (IMBASE + 0xD00 + (slot *2)));
	cmdRamPtr = &SPICMDARRAY[slot];	
	WriteAXReg(0x20, 0xC7);  // power on, enable all axes
	RTCDelayTicks(10000l);  // wait one quarter second
	WriteAXReg(0x21, 0x04);  // set command register to enable DRDY
	
	// initialize the AQQ also
	AXQinidx = 0;
	AXQoutidx = 0;
	AXQlength = 0;
	AXActive = false;

}

void ShowAXRegisters(void){
	cprintf("/nAccelerometer register values\n");
	cprintf("0x0F  %02X\n", ReadAXReg(0x0f));
	cprintf("0x16  %02X\n", ReadAXReg(0x16));
	cprintf("0x17  %02X\n", ReadAXReg(0x17));
	cprintf("0x18  %02X\n", ReadAXReg(0x18));
	cprintf("0x19  %02X\n", ReadAXReg(0x19));
	cprintf("0x1A  %02X\n", ReadAXReg(0x1A));
	cprintf("0x1B  %02X\n", ReadAXReg(0x1B));
    // Control Registers
	cprintf("0x20  %02X\n", ReadAXReg(0x20));
	cprintf("0x21  %02X\n", ReadAXReg(0x21));
	cprintf("0x22  %02X\n", ReadAXReg(0x22));
	cprintf("0x27  %02X\n", ReadAXReg(0x27));
		
}


void WriteAXReg(unsigned char reg, unsigned char val){
unsigned short cmd;
	// msb of command is 0 for write
	cmd = val + ((reg & 0x3F) << 8);
	*cmdRamPtr = 0x4B;
	//cprintf("\nWrite Command 0x%04X  ",cmd);
	*cmdptr = cmd;	
	while(*SPSR < 128){ }; // wait until qsm done flag set
	
	*SPCR2 =(axslot<<8) + axslot;
	*SPSR = 0;
	*SPCR0 = 0x8006 | CPOL | CPHA; 	// sets master, 16 bits  baud divisor 8, CPHA 1
	*SPCR1 |= 0x8000;   // start qsm to get last result
	Delay2us();
	while(*SPSR < 128){ }; // wait until qsm done flag set
}


// read a single register with setup
unsigned char ReadAXReg(unsigned char reg){
short cmd;
unsigned char rdata;
	// msb of command is 1 for read
	cmd = 0x8000 + ( (reg & 0x3F) << 8);
	*cmdRamPtr = 0x4B;  // only PCS2 low

	*cmdptr = cmd;	
	while(*SPSR < 128){ }; // wait until qsm done flag set
	
	*SPCR2 =(axslot<<8) + axslot;  // set start and end queue the same
	*SPSR = 0;
	*SPCR0 = 0x8006 | CPOL | CPHA; 	// sets master, 16 bits  baud divisor 16, CPHA 1
	*SPCR1 |= 0x8000;   // start qsm to get last result
	Delay2us();
	while(*SPSR < 128){ }; // wait until qsm done flag set
	
	
	rdata = (*rcvdatptr) & 0xFF;   	 

	return rdata;
}

// read more registers without setup
unsigned char ReadMoreReg(unsigned char reg){
short cmd;
unsigned char rdata;
	// msb of command is 1 for read
	cmd = 0x8000 + ( (reg & 0x3F) << 8);

	*cmdptr = cmd;	
//	while(*SPSR < 128){ }; // wait until qsm done flag set
	
//	*SPCR2 =(axslot<<8) + axslot;  // set start and end queue the same
	*SPSR = 0;
	*SPCR1 |= 0x8000;   // start qsm to get last result
	Delay2us();
	while(*SPSR < 128){ }; // wait until qsm done flag set
	
	
	rdata = (*rcvdatptr) & 0xFF;   	 

	return rdata;
}


//  Called from PIT Chore  reads the accelerometer results and
//  stores in the global variables. Also puts data in AXQueue
void ReadAccel(void){
unsigned char lo,hi;
register AXQElem *aqp;

unsigned long secs;
unsigned short ticks;

	if(AXActive){  // don't collect unless flag is set
		lo = ReadAXReg(OUTX_L);  
		hi = ReadMoreReg(OUTX_H); 
		ax = (hi <<8) + lo; 
		lo = ReadMoreReg(OUTY_L);  	
		hi = ReadMoreReg(OUTY_H); 
		ay = (hi <<8) + lo; 
		lo = ReadMoreReg(OUTZ_L);  
		hi = ReadMoreReg(OUTZ_H);  	
		az = (hi <<8) + lo;
		RTCGetTime(&secs, &ticks);
		// now put the time and acceleration values into the queue
		if(AXQlength < AXQMAX){
			aqp = &AXQData[AXQinidx];
			aqp->qsec = secs;	
			aqp->qtick = ticks;			
			aqp->qax = ax;			
			aqp->qay = ay;
			aqp->qaz = az;
			AXQinidx++;
			if(AXQinidx >= AXQMAX) AXQinidx = 0;
			AXQlength++;
					
		}

	}
}


//  called by foreground loop to pull queue elements and write to file
void CheckAccelQueue(char *b){
register AXQElem *aqp;
FILE *f;
float aax, aay,aaz;
char tms[30];
short tsec;
struct tm *tmp;
unsigned long secs;
unsigned short ticks;
	
	if(AXQlength > 0){
		aqp = &AXQData[AXQoutidx];
		if(axfile != NULL){
			fwrite(aqp, 1,sizeof(AXQElem), axfile);
		}
			RTCGetTime(&secs, &ticks);
			aqp->qsec = secs;	
			aqp->qtick = ticks;			
			aqp->qax = ax;			
			aqp->qay = ay;
			aqp->qaz = az;
		
			f = fopen(b,"a");
			fflush(f);
			tsec = (ticks+200)/400;
			tmp =localtime(&secs);
			strftime(tms, 31," %H:%M:%S",tmp);
			aax=ax* AXSCALE + AXOFFSET;
			aay=ay* AYSCALE + AYOFFSET;
			aaz=az* AZSCALE + AZOFFSET;
			fprintf(f,"%s.%02d, %4.2f, %4.2f, %4.2f\n",tms,tsec,aax,aay,aaz);
			fclose(f);
			
		AXQoutidx++;
		if(AXQoutidx >= AXQMAX) AXQoutidx = 0;
		AXQlength--;
		axtotal++;		
	}
}

void AccelStart(void){
	AXQinidx = 0;
	AXQoutidx = 0;
	AXQlength = 0;
	axtotal = 0;
	AXActive = false;
	//axfile = fopen("axtemp.dat", "wb");
	//cprintf("Opened axtemp.dat\n");
	AXActive = true;

}

void AccelStop(void){
	AXActive = false;
	if(axfile != NULL){
		fclose(axfile);
		cprintf("Closed axtemp.dat.   %ld values were written\n", axtotal);
		axfile = NULL;
	}

}


void AccelPlayback(void){
char *pbs;
bool done = false;

	AXActive = false;
	if(axfile != NULL){
		fclose(axfile);
		axfile = NULL;
	}	
	if(AXOpenPlayback()){
	
		cprintf("\n");  // put header here
		while(!done){
			pbs = AXGetPlayback();
			if(pbs[0] == 0) done = true;
			printf("%s\n",pbs);
		}	
	} else {
		cprintf("\nCan't open axtemp.dat\n");	
	}
	cprintf("\n");
}


bool AXOpenPlayback(void){

	axfile = fopen("axtemp.dat", "rb");
	if(axfile == NULL){
		cprintf("\nCan't open axtemp.dat\n");
		return false;		
	} else return true;

}

// return a string with the ADC portion of a playback
char  *AXGetPlayback(void){
static char pbs[100];
char tms[30];

short tsec, numread;
float fax,fay,faz;
struct tm *tmp;
AXQElem aqe;

	if(axfile == NULL){ // error exit if file not open
		pbs[0] = 0;
		return &pbs[0];
	}
	
	if(!feof(axfile) && !kbhit()){
		numread = fread(&aqe, 1, sizeof(aqe), axfile);
		if(numread == sizeof(aqe)){
			tsec =  (aqe.qtick+200)/400;
			tmp =localtime(&aqe.qsec);
			strftime(tms, 31," %H:%M:%S",tmp);
			fax = aqe.qax * AXSCALE + AXOFFSET;
			fay = aqe.qay * AYSCALE + AYOFFSET;		
			faz = aqe.qaz * AZSCALE + AZOFFSET;	
#ifdef USECOMMAS
			sprintf(pbs,"%s.%02d, %4.2f, %4.2f, %4.2f",tms,tsec, fax,fay,faz);
#else
			//sprintf(pbs," %s.%02d %4.2f %4.2f %4.2f",tms,tsec, fax,fay,faz);			
#endif	

        } else {
            pbs[0] = 0;
        }
	
	} else {
		pbs[0] = 0;
		fclose(axfile);
		axfile = NULL;
	}
	return &pbs[0];


}