//*****************************************************************************/
//  Calibration Constants for IODP data logging program
//  M. Borgerson     10/29.2007
//
//  Calibration constants are kept in this file to preclude
//  inadvertent changes to the constants
//
//*****************************************************************************/


// if USETABS is defined, tab characters are used as separators
// in the data display.  Un-Comment the define statement
// and recompile for tabs

//#define USETABS
#define USECOMMAS


// The following lines define string constants displayed in the
// file header.  They must be enclosed in quotation marks.
#define  TOOLNUMBER    "Not Entered"
#define  PTSERNUM      "333"
#define  PROGVERSION   "V1.0"
#define  LASTCALDATE   "11/13/2009"

#define  LOWBATTERY    7.0

// constants for the conversion of accelerometer values to Gs
//
//  Sample calculation:  fax = aqe.qax * AXSCALE + AXOFFSET;
//    where qax is the short integer counts recorded in temp file

#define AXSCALE  (2.0/2048)
#define AXOFFSET  0

#define AYSCALE  (2.0/2048)
#define AYOFFSET  0

#define AZSCALE  (2.0/2048)
#define AZOFFSET  0


//*****************************************************************************/
// ADC and tip temperature calibration constants
//
#define RTIPSCALE   33000.0
#define RTIPOFFSET  0.0

#define TIPCALOFFSET  0xFFFB0500
#define TIPCALSCALE   0x0107F8A2

#define TTA     0.001124
#define TTB     0.000235
#define TTC     8.45e-8

//*****************************************************************************/
// ADC and board temperature calibration constants
//
#define RBRDSCALE   (10000.0)
#define RBRDOFFSET  0.0

#define BRDCALOFFSET  0xFFFAC000
#define BRDCALSCALE   0x01B3881A

#define BTA     0.001404166
#define BTB     0.000237082
#define BTC     1.007e-7
//*****************************************************************************/
// ADC and Battery voltage calibration constants
//
#define VBATSCALE   10.0
#define VBATOFFSET   0.0

#define BATCALOFFSET  0xFFFAE600
#define BATCALSCALE   0x02DC2564
//*****************************************************************************/
// QSPI constants.  Do not change these values
//

#define IMBASE          0xFFFFF000
#define     QMCR            ((uspv) (IMBASE + 0xC00))
#define     QILR            ((ucpv) (IMBASE + 0xC04))
#define     QVLR            ((ucpv) (IMBASE + 0xC05))
#define     QPDR            ((ucpv) (IMBASE + 0xC15))
#define     QPAR            ((ucpv) (IMBASE + 0xC16))
#define     QDDR            ((ucpv) (IMBASE + 0xC17))
#define     SPCR0           ((uspv) (IMBASE + 0xC18))
#define     SPCR1           ((uspv) (IMBASE + 0xC1A))
#define     SPCR2           ((uspv) (IMBASE + 0xC1C))
#define     SPCR3           ((ucpv) (IMBASE + 0xC1E))
#define     SPSR            ((ucpv) (IMBASE + 0xC1F))
#define     SPICMD          ((ucpv) (IMBASE + 0xD40))
#define     SPICMDARRAY     ((uchar *) (IMBASE + 0xD40))


typedef volatile uchar      *ucpv;
typedef unsigned int        unsint;
typedef volatile ushort     *uspv;
typedef volatile ulong      *ulpv;
