/******************************************************************************\
**	IODP_Paro.c		IODP ParoScientific Pressure sensor driver
**	
*****************************************************************************
**	    Copyright 2007   Oceanographic Embedded Systems
**	
*****************************************************************************
**	
**	
\******************************************************************************/


#include	<cfxbios.h>		// Persistor BIOS and I/O Definitions
#include	<cfxpico.h>		// Persistor PicoDOS Definitions
#include	<assert.h>
#include	<ctype.h>
#include	<errno.h>
#include	<float.h>
#include	<limits.h>
#include	<locale.h>
#include	<math.h> 
#include	<setjmp.h>
#include	<signal.h>
#include	<stdarg.h>
#include	<stddef.h>
#include	<stdio.h> 
#include	<stdlib.h>
#include	<time.h>
#include 	<string.h>

#include   "IODP_Paro.h"
#include   "IODP_CalConstant.h"


TUPort *tuport;
TUChParams *ptup;


int GetSerLine(char *s, int maxlen);




//*****************************************************************************/
// get serial input line from TPU UART port
//*****************************************************************************/
int GetSerLine(char *s, int maxlen){
short count;
char   *sptr;
int i;
  sptr = s;

  *sptr = 0;
  count = 0;
  count = TURxQueuedCount(tuport);
  if(count > maxlen-2) count = maxlen-2;
  for(i=0; i< count; i++){
       *sptr++ = TURxGetByte(tuport, false); 
    }   
    *sptr = 0x00;   // terminate the string
    
  return count;  
 }
  

//*****************************************************************************/
//  Read data from pressure sensor
//*****************************************************************************/


void   InitPressure(void){
char str[30];
	PIORead(48);  //Set RTS pin as input
	PinRead(37);   //this becomes TPU UART input
	PIOSet(35);   //output high makes RS-232 RTS low
	TUInit(calloc, free);
	ptup = TUGetDefaultParams();
	//cprintf("Opening TPU UART for pressure sensor.\n");
	
	tuport = TUOpen(15, 14, 9600l, NULL);

	if (tuport == 0) {
		cprintf("\n!!! Unexpected error opening TPU UART channel.\n");
		return;
	}
	RTCDelayMicroSeconds(100000l);
	TUTxPrintf(tuport,"*0100PR\n"); // Check resolution
	
	RTCDelayMicroSeconds(200000l);
    GetSerLine(&str[0], 28);
//    cprintf("Pressure Resolution: %s", str);	
}

void   RequestPressure(void){
	TURxFlush(tuport);  // get rid of leftover characters
    TUTxPrintf(tuport,"*0100P3\n"); //  collect and send one pressure reading

}


// This routine took 10msec before optimization
char *ParsePressure(void){
static char str[30];
char *pptr;
int len;
	str[5] = 0;
    GetSerLine(&str[0], 28);
	pptr = &str[5];
	len = strlen(pptr);
	// now trim trailing CR or LF from string
	if(pptr[len-1] < '0') pptr[len-1] = 0;
	if(pptr[len-2] < '0') pptr[len-2] = 0;
	return pptr;
}


