/******************************************************************************\
**	SLEEP.c			IODP Data Collection Program with Lowered Sampling Rates
**	
*****************************************************************************
**	    Copyright 2007   Oceanographic Embedded Systems
**	
*****************************************************************************
**	
**	
\******************************************************************************/

#include	<cfxbios.h>		// Persistor BIOS and I/O Definitions
#include	<cfxpico.h>		// Persistor PicoDOS Definitions

#include	<assert.h>
#include	<ctype.h>
#include	<errno.h>
#include	<float.h>
#include	<limits.h>
#include	<locale.h>
#include	<math.h>
#include	<setjmp.h>
#include	<signal.h>
#include	<stdarg.h>
#include	<stddef.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<time.h>

#include	<dirent.h>		// PicoDOS POSIX-like Directory Access Defines
#include	<dosdrive.h>	// PicoDOS DOS Drive and Directory Definitions
#include	<fcntl.h>		// PicoDOS POSIX-like File Access Definitions
#include	<stat.h>		// PicoDOS POSIX-like File Status Definitions
#include	<termios.h>		// PicoDOS POSIX-like Terminal I/O Definitions
#include	<unistd.h>		// PicoDOS POSIX-like UNIX Function Definitions

#include "IODP_CalConstant.h"
#include "IODP_Accel.h"
#include "IODP_ADC.h"
#include "CS5534Lib.h"

unsigned char axstat;
char *FRefString[60];

void ShowHeader(FILE *f);
void ShowInformation(void);
void VerifyDate(void);
void GetHeader(void);
void ShowCommands(void);
void ConvertFile(void);
void PITChore(void);
void PlayBoth(void);


void GetMem(float,float);  //Seven Variables created by K. Darnell 04/10
char outname[7];
char fadcout[11];
char faxout[10];
int x = 1, y = 2;
float *axrate;
float *adcrate;
float axgap=1,adcgap=1;



//*****************************************************************************/
// PIT Chore runs at 50Hz.   It schedules data collection
//*****************************************************************************/
void PITChore(void){
static short PITCount = 0;
static short count=0;

if(PITCount==0){
	count++;
}

	// pit chore can't do anything taking more than about 2msec
	// reading 6 bytes of accel data takes about 160 microseconds before optimization

	PinSet(1);
	
	//Change routine sampling intervals based on user preference - K. Darnll 04/10
	
	if((axgap < 1) && ((PITCount % 5) == 0)){
	
 	   		ReadAccel();

			}
			
	else{
	if((axgap>1) && (count % (int)axgap ==0) && !(axgap == 1.0/0.0)){
			if(PITCount ==5){ ReadAccel();
				}
		}
				
				
	if(((count % (int)adcgap) == 0) && !(adcgap == 1.0/0.0)){
	ReadADC(PITCount);
	}
	}
	


	PITCount++;
	if(PITCount > 49) PITCount = 0;

	PinClear(1);
	
}


//******************************************************************************
// Play back the formatted data from the binary files
// Data is sent to the serial port
//******************************************************************************
void PlayBoth(void){
char *axp, *adp;
bool done = false;

	ShowHeader(stdout);
	fflush(stdout);
	AXOpenPlayback();
	ADCOpenPlayback();
	while(!done){
		axp = AXGetPlayback();
		adp = ADCGetPlayback();
		if((*adp == 0) && (*axp == 0)){
			done = true;
		} else {
			if(*adp == 0){  // fill start of line
#ifdef USECOMMAS
			    putstr(", , , , , , ");

#else
			    putstr("                                                        ");
#endif 

			} else {
				putstr(adp);
			}
			putstr(axp);
			

			putch('\r');
		}
	
	}
}


// This function will print the header information---either to the
// console (f = stdout), or to an ascii file (f = outfile);
void ShowHeader(FILE *f){
struct tm *tmp;
time_t now;
char tms[40];
//char *ToolString;
	//fstart = ADFileStart();
	now =time(NULL);
	fflush(f);
	fprintf(f,"\n");
	tmp = localtime(&now);
	strftime(tms, 39,"%m/%d/%y ",tmp);
	//ToolString = VEEFetchStr("TOOLSN", TOOLNUMBER);
	fprintf(f," ACQUISITION DATE:,,, %s\n",tms);
	fprintf(f," CF-2 SERIAL NUMBER:,,, %ld\n",BIOSGVT.CFxSerNum);
	fprintf(f," PROGRAM VERSION:,,, %s\n",PROGVERSION);
	//fprintf(f," LAST CALIBRATION DATE:,,, %s\n", LASTCALDATE);
	fprintf(f," FILE REFERENCE:,,, %s\n", FRefString);
	//fprintf(f," PRESSURE TRANSDUCER S/N:,,, %s\n",PTSERNUM);
	fprintf(f,"\n\n");
	
	// now do the dat column headers
#ifdef USECOMMAS
	fprintf(f," GMT TIME,");
	fprintf(f," Tip Value, Brd Temp, Batt Volts, U7-Ch. 1, U7-Ch. 2\n");
	fprintf(f," GMT Time  , X  , Y , Z\r"); 
	fprintf(f," HH:MM:SS  , Counts,  deg C , Volts ,  Counts,   Counts\n");
	//fprintf(f," HH:MM:SS  , g  , g , g\n");     
#else
	fprintf(f,"");
	fprintf(f," GMT Time   Tip Temp Tip R     Brd Temp Battery  Pressure ");
	fprintf(f," GMT Time     X    Y    Z\n");  
	fprintf(f," HH:MM:SS      deg C  Ohms      deg C   Volts        PSI ");
	fprintf(f,"  HH:MM:SS     g    g    g\n");  
#endif			 

}

// Display some information about system status
void ShowInformation(void){
struct tm *tmp;
time_t tnow;
char tms[40];


	tnow = time(NULL);

	cprintf("\n");
	tmp = localtime(&tnow);
	strftime(tms, 39,"%m/%d/%y  %H:%M:%S",tmp);
	cprintf("CURRENT TIME AND DATE: %s\n", tms);
	cprintf("      PROGRAM VERSION: %s\n",PROGVERSION);
	//cprintf("LAST CALIBRATION DATE: %s\n", LASTCALDATE);
	cprintf("       FILE REFERENCE: %s\n", FRefString);
	cprintf("\n\n");
}



// Show date and time.  Ask user for new date and time if needed
void VerifyDate(void){
struct tm *tmp;
time_t now,newtime;
char tms[40];
	cprintf("\nThe CF-2 date and time are: ");
	now = time(NULL);
	tmp = localtime(&now);
	strftime(tms, 39,"%m/%d/%y  %H:%M:%S",tmp);
	cprintf("%s\n",tms);
	if(!QRconfirm("Are these values correct ",true,true)){
		cprintf("\n");
		if(QRdatetime("Enter date and time: ",MMDDYY, false,true, tmp)){
			newtime = mktime(tmp);
			RTCSetTime(newtime, 0);
			cprintf("\nCF-2 time and date have been adjusted.\n");
		}
	} else cprintf("\n");
	
}

// Get some information about this logging session
void GetHeader(void){
time_t tnow;
float bvolts;

	bvolts = GetVolts();
	if(bvolts < LOWBATTERY){
		cprintf("\n************************* WARNING *****************************\n");
		printf( "Battery voltage of %7.2f is below minimum for data collection\n",bvolts);
		cprintf("************************* WARNING *****************************\n");	
	}
	// wait until the second changes to exit

	cprintf("\nStarting Data Collection.\n");
	tnow = time(NULL);
	while(tnow != time(NULL)){}
		
}

// Show the commands available
void ShowCommands(void){
	cprintf("Commands: \n");
	cprintf("d :  Adjust date and time    e :  End Data Collection \n");
	cprintf("s :  Start Data Collection   i :  Show system Information \n");
	cprintf("p :  Play Back data          ? :  Show this command list \n");
	cprintf("q :  Quit program \n");
	
}

//This routine is obselete - K. Darnell 04/10


//******************************************************************************
// Convert the temporary binary files to a single ASCII file on CF card
//
//******************************************************************************
void ConvertFile(void){
char *axp, *adp;
char fname[14];
bool done = false;
FILE *outfile;
long linecount = 0;

	AXOpenPlayback();
	ADCOpenPlayback();
	strcpy(fname, "IODPTEMP.TXT");
	cprintf("\n");
	if(!QRstring("Name for converted file ","%s",true, fname, 11))return;
	
	outfile = fopen(fname,"w");
	if(outfile == NULL){
		cprintf("Unable to open file %s\n", fname);
		return;
	}
	cprintf("\nStarting conversion.\n");
	ShowHeader(outfile);
	while(!done){
		axp = AXGetPlayback();
		linecount++;
		if((linecount %60) == 0){
			putch('.');
			fflush(stdout);
		}
		if((linecount % 3600) == 0){
			putch('\n');
			fflush(stdout);
		}
		adp = ADCGetPlayback();
		if((*adp == 0) && (*axp == 0)){
			done = true;
		} else {
			if(*adp == 0){  // fill start of line
#ifdef USECOMMAS
			    fputs(", , , , , , ",outfile);
#else
			    fputs("            	         	          	        	  ",outfile);

#endif
	       	} else {
				fputs(adp, outfile);
			}
			fputs(axp,outfile);
			fputs("\n", outfile);
		}
	
	} // end of while(!done)
	fclose(outfile);
}

/************************************************************************
** Function for Determining Logging Time Available - K. Darnell 04/10
************************************************************************/

void GetMem(float axg, float adcg){
long secs, space;
int  headers, onereadax, onereadadc,hours, minutes, seconds, days;

//Free space on Volume
space = DIRFreeSpace("C:/");
printf("\n\nThere are %li bytes available on this device.\n",space);

//Number of Bytes to perform one second of logging
headers = 608;
onereadax = 340;
onereadadc = 58;

if(axg/=0.1){
	secs = (space - headers)/(onereadax/axg/10+onereadadc/adcg);
	}
else{
	secs = (space - headers)/(onereadax+onereadadc/adcg);	
	}

seconds = secs%60;
minutes = (secs/60)%60;
hours = secs/3600%24;
days = secs/3600/24;
printf("Therefore you have:\n");
printf("%d days %d hours %d minutes and %d seconds of logging time left.\n\n",days, hours, minutes, seconds);

}


/******************************************************************************\
**	main
\******************************************************************************/

int main(int argc, char **argv) {
#pragma unused(argc, argv)
short result = 0;
char ch;
long tnow;
float bvolts;
bool done = false;
bool acqrunning = false;
bool acqhasrun = false;
FILE *fadc,*fax;


	SCITxSetBuffered(true);  // use interrupt-driven serial I/O
	SCIRxSetBuffered(true);	
		
	
	// Identify the progam and build
	printf("\nProgram: %s: %s %s \n", __FILE__, __DATE__, __TIME__);
	// Identify the device and its firmware
	printf("Persistor CF%d SN:%ld   BIOS:%d.%d   PicoDOS:%d.%d\n", CFX,
		BIOSGVT.CFxSerNum, BIOSGVT.BIOSVersion, BIOSGVT.BIOSRelease, 
		BIOSGVT.PICOVersion, BIOSGVT.PICORelease);


	strcpy(FRefString, "Not Entered.");
	PIOClear(1);

	TMGSetSpeed(16000);	

	PITRemoveChore(NULL);
	PITSet100usPeriod(PITOff);
	
	cprintf("Enabling Accelerometers");
	InitAXSPI(4);  // Use QSPI slot 4
	InitADSPI(8);  // Use two QSPI slots starting at slot 8
			          
	PITAddChore(&PITChore,6);   // use interrupt priority 6
	// sync the PIT counter to the RTC
	tnow = time(NULL);
	while(time(NULL) == tnow){}
    // Start the PIT timer just after the second changes
	PITSet100usPeriod(PIT50Hz); 
	bvolts = GetVolts();
	
	VerifyDate();  // make sure CF-2 date and time are correct
	QRstring("File Reference ","%s",false, FRefString, 60);
	
	//Create an output file to write directly to - K. Darnell 04/10
	QRstring("\nOutput File (limit to 7 characters!)","%s",false, outname, 7);
	
	
	//Request sampling rates from user - K. Darnell 04/10
	
	axrate = &x;
	adcrate = &y;
	
	if(QRconfirm("\n\nSample from Accelerometer ",true,true)){
		cprintf("\nAccelerometer regularly samples at 10Hz");
		cprintf("\nFor longer sampling times, enter appropriate sampling rate.");
		Delay2us();
		QRfloat("\nWhat sampling rate [Hz]","%f",false, axrate,0.00000001,10);
		}
	else{
	*axrate = 0;
	}
	
	axgap = *axrate;
	axgap = 1.0/axgap;
	
	if(QRconfirm("\n\nSample from ADC ",true,true)){
		cprintf("\nAnalog to Digital converter regularly samples at 1Hz.");
		cprintf("\nFor longer sampling times, enter appropriate sampling rate.");
		Delay2us();
		QRfloat("\nWhat sampling rate [Hz]","%f",false, adcrate,0.00000001,1);
		}
	else{
	*adcrate = 0;
	}
	
	adcgap = *adcrate;
	adcgap = 1.0/adcgap;
	
	if(!(axgap == 1.0/0)){
	
		printf("\n\nThe accelerometer interval period is %5.1f seconds.",axgap); 
	}
	
	else{
	
		printf("\n\nThe accelerometer is not sampling.",axgap);
	}
	
	if(!(adcgap == 1.0/0)){
		printf("\nThe ADC interval period is %5.1f seconds.",adcgap);
		}
	else{
	
		printf("\n\nThe Analog to Digital converter is not sampling.",axgap);
	}
	
	//Display available memory based on user responses - K. Darnell
	
	GetMem(axgap,adcgap);
	
	strcpy(fadcout,outname);
	strcat(fadcout,"D");
	
	
	strcpy(faxout,outname);
	strcat(faxout,"X");
	
	// now get the file reference string
	
	
	ShowInformation( );  // Show the serial number, etc.
	ShowCommands();
	
	
	bvolts = GetVolts();
	if(bvolts < LOWBATTERY){
		cprintf("\n************************* WARNING *****************************\n");
		printf( "Battery voltage of %7.2f is below minimum for data collection\n",bvolts);
		cprintf("************************* WARNING *****************************\n");	
	}
	cprintf("CMD: "); fflush(stdout);
	
	// 11/27/07  if acquisition is running, only 'e' is valid command
	//           after first 's' command, you cannot do another
	while(!done){
	    CheckAccelQueue(faxout); // Pass Accelerometer File Name - K. Darnell 04/10
	    CheckADCQueue();
	    if(UpdateFlag) ShowADC(fadcout); // Pass Analog to Digital Converter File Name - K. Darnell 04/10
		if(kbhit()){
			ch = getch();
			ch = tolower(ch);
			switch(ch){
				case '1' 	: if(!acqrunning) ShowADCValue(1);
							break;
				case '2' 	:if(!acqrunning) ShowADCValue(2);
							break;
				case '3' 	: if(!acqrunning)ShowADCValue(3);
							break;
				case '4' 	: if(!acqrunning)ShowADCValue(4);
							break;
				case '5' 	: if(!acqrunning) ShowADCValue(5);
							break;
				case '6' 	:if(!acqrunning) ShowADCValue(6);
							break;
				case '7' 	: if(!acqrunning)ShowADCValue(7);
							break;
				case '8' 	: if(!acqrunning)ShowADCValue(8);
							break;
				case 'd' 	: if(!acqrunning)VerifyDate();
							break;

				case 'a' 	: if(!acqrunning)AccelPlayback();
							break;
				case 's' 	: if(!acqhasrun){
								acqrunning = true;
								acqhasrun = true;
							  	GetHeader();
							  	fadc = fopen(fadcout,"w");
								ShowHeader(fadc);
								fclose(fadc);
								
								fax = fopen(faxout,"w");
								ShowHeader(fax);
								fclose(fax);
								
							  	AccelStart(); 
							  	ADCStart();  	 
							  } else cprintf("Acquisition can be started only oncer per session.\n");							  
							break;
				case 'e' 	: ADCStop();  AccelStop(); acqrunning = false;
							break;
				//case 'f' 	:if(!acqrunning) ConvertFile();
							//break;							
				case 'q' 	:ADCStop();  AccelStop(); 
							done = true;
							break;
				case 'p' 	: if(!acqrunning)PlayBoth();
							break;
				case 'i' 	:if(!acqrunning) ShowInformation(); 
							break;							
				case '?'    : if(!acqrunning)ShowCommands();
							break;
				default : cprintf("\n"); 
							break;
														
			} // end of switch(ch)
			cprintf("\nCMD: "); fflush(stdout);
		};  // end of if(kbhit()).....
	}	// end  of if(while(!done)...

	PITSet100usPeriod(PITOff); 	
	
// Various exit strategies
//	BIOSReset();			// full hardware reset
//	BIOSResetToPBM();		// full reset, but stay in Pesistor Boot Monitor
	BIOSResetToPicoDOS();	// full reset, but jump to 0xE10000 (PicoDOS)
	return result;

	}	//____ main() ____//

